@@ -8,11 +8,11 @@ from composeml.tests.utils import to_csv
 def test_search_default(transactions, total_spent_fn):
     lm = LabelMaker(target_entity='customer_id', time_index='time', labeling_function=total_spent_fn)
 
-    given_labels = lm.search(transactions, num_examples_per_instance=1, verbose=False)
+    given_labels = lm.search(transactions, num_examples_per_instance=1)
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '0,2019-01-01 08:00:00,2',
         '1,2019-01-01 09:00:00,3',
         '2,2019-01-01 10:30:00,4',
@@ -22,6 +22,85 @@ def test_search_default(transactions, total_spent_fn):
     assert given_labels == labels
 
 
+def test_search_examples_per_label(transactions, total_spent_fn):
+    def total_spent(ds):
+        return total_spent_fn(ds) > 2
+
+    lm = LabelMaker(target_entity='customer_id', time_index='time', labeling_function=total_spent)
+
+    n_examples = {True: -1, False: 1}
+    given_labels = lm.search(transactions, num_examples_per_instance=n_examples, gap=1)
+    given_labels = to_csv(given_labels, index=False)
+
+    labels = [
+        'customer_id,time,total_spent',
+        '0,2019-01-01 08:00:00,False',
+        '1,2019-01-01 09:00:00,True',
+        '1,2019-01-01 09:30:00,False',
+        '2,2019-01-01 10:30:00,True',
+        '2,2019-01-01 11:00:00,True',
+        '2,2019-01-01 11:30:00,False',
+        '3,2019-01-01 12:30:00,False',
+    ]
+
+    assert given_labels == labels
+
+
+def test_search_with_undefined_labels(transactions, total_spent_fn):
+    def total_spent(ds):
+        return total_spent_fn(ds) % 3
+
+    lm = LabelMaker(target_entity='customer_id', time_index='time', labeling_function=total_spent)
+
+    n_examples = {1: 1, 2: 1}
+    given_labels = lm.search(transactions, num_examples_per_instance=n_examples, gap=1)
+    given_labels = to_csv(given_labels, index=False)
+
+    labels = [
+        'customer_id,time,total_spent',
+        '0,2019-01-01 08:00:00,2',
+        '0,2019-01-01 08:30:00,1',
+        '1,2019-01-01 09:30:00,2',
+        '1,2019-01-01 10:00:00,1',
+        '2,2019-01-01 10:30:00,1',
+        '2,2019-01-01 11:30:00,2',
+        '3,2019-01-01 12:30:00,1',
+    ]
+
+    assert given_labels == labels
+
+
+def test_search_with_multiple_targets(transactions, total_spent_fn, unique_amounts_fn):
+    lm = LabelMaker(
+        target_entity='customer_id',
+        time_index='time',
+        window_size=2,
+        labeling_function={
+            'total_spent': total_spent_fn,
+            'unique_amounts': unique_amounts_fn,
+        },
+    )
+
+    actual = lm.search(
+        transactions,
+        num_examples_per_instance=-1,
+    )
+
+    actual = to_csv(actual, index=False)
+
+    expected = [
+        'customer_id,time,total_spent,unique_amounts',
+        '0,2019-01-01 08:00:00,2,1',
+        '1,2019-01-01 09:00:00,2,1',
+        '1,2019-01-01 10:00:00,1,1',
+        '2,2019-01-01 10:30:00,2,1',
+        '2,2019-01-01 11:30:00,2,1',
+        '3,2019-01-01 12:30:00,1,1',
+    ]
+
+    assert actual == expected
+
+
 def test_search_offset_mix_0(transactions, total_spent_fn):
     """
     Test offset mix with window_size (absolute), minimum_data (absolute), and gap (absolute).
@@ -39,13 +118,12 @@ def test_search_offset_mix_0(transactions, total_spent_fn):
         minimum_data='30min',
         gap='2h',
         drop_empty=True,
-        verbose=False,
     )
 
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '0,2019-01-01 08:30:00,1',
         '1,2019-01-01 09:30:00,2',
         '2,2019-01-01 11:00:00,3',
@@ -70,13 +148,12 @@ def test_search_offset_mix_1(transactions, total_spent_fn):
         num_examples_per_instance=2,
         minimum_data='2019-01-01 10:00:00',
         gap='4h',
-        verbose=False,
     )
 
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '1,2019-01-01 10:00:00,1',
         '2,2019-01-01 10:00:00,4',
         '3,2019-01-01 10:00:00,1',
@@ -100,13 +177,12 @@ def test_search_offset_mix_2(transactions, total_spent_fn):
         transactions,
         num_examples_per_instance=2,
         minimum_data=2,
-        verbose=False,
     )
 
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '1,2019-01-01 10:00:00,1',
         '2,2019-01-01 11:30:00,1',
         '2,2019-01-01 12:00:00,1',
@@ -131,13 +207,12 @@ def test_search_offset_mix_3(transactions, total_spent_fn):
         num_examples_per_instance=-1,
         minimum_data='2019-01-01 08:00:00',
         gap=1,
-        verbose=False,
     )
 
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '0,2019-01-01 08:00:00,2',
         '0,2019-01-01 08:30:00,1',
         '1,2019-01-01 09:00:00,3',
@@ -168,13 +243,12 @@ def test_search_offset_mix_4(transactions, total_spent_fn):
         transactions,
         num_examples_per_instance=2,
         gap='30min',
-        verbose=False,
     )
 
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '0,2019-01-01 08:00:00,1',
         '0,2019-01-01 08:30:00,1',
         '1,2019-01-01 09:00:00,1',
@@ -203,13 +277,12 @@ def test_search_offset_mix_5(transactions, total_spent_fn):
         num_examples_per_instance=2,
         minimum_data='1h',
         gap=2,
-        verbose=False,
     )
 
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '1,2019-01-01 10:00:00,1',
         '2,2019-01-01 11:30:00,2',
     ]
@@ -233,13 +306,12 @@ def test_search_offset_mix_6(transactions, total_spent_fn):
         num_examples_per_instance=1,
         minimum_data=3,
         gap=1,
-        verbose=False,
     )
 
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '2,2019-01-01 12:00:00,1',
     ]
 
@@ -261,13 +333,12 @@ def test_search_offset_mix_7(transactions, total_spent_fn):
     given_labels = lm.search(
         transactions,
         num_examples_per_instance=float('inf'),
-        verbose=False,
     )
 
     given_labels = to_csv(given_labels, index=False)
 
     labels = [
-        'customer_id,cutoff_time,total_spent',
+        'customer_id,time,total_spent',
         '0,2019-01-01 08:00:00,2',
         '1,2019-01-01 09:00:00,3',
         '2,2019-01-01 10:30:00,4',
@@ -293,7 +364,6 @@ def test_search_offset_negative_0(transactions, total_spent_fn):
             num_examples_per_instance=2,
             minimum_data=-1,
             gap=-1,
-            verbose=False,
         )
 
 
@@ -313,7 +383,6 @@ def test_search_offset_negative_1(transactions, total_spent_fn):
             num_examples_per_instance=2,
             minimum_data='-1h',
             gap='-1h',
-            verbose=False,
         )
 
 
@@ -356,7 +425,6 @@ def test_invalid_threshold(transactions, total_spent_fn):
             transactions,
             num_examples_per_instance=2,
             minimum_data=' ',
-            verbose=False,
         )
 
 
@@ -368,13 +436,13 @@ def test_search_invalid_n_examples(transactions, total_spent_fn):
     )
 
     with pytest.raises(AssertionError, match='must specify gap'):
-        next(lm.slice(transactions, num_examples_per_instance=2, verbose=False))
+        next(lm.slice(transactions, num_examples_per_instance=2))
 
     with pytest.raises(AssertionError, match='must specify gap'):
-        lm.search(transactions, num_examples_per_instance=2, verbose=False)
+        lm.search(transactions, num_examples_per_instance=2)
 
 
-def test_search_empty_labels(transactions, total_spent_fn):
+def test_search_on_empty_data_slices(transactions, total_spent_fn):
     lm = LabelMaker(
         target_entity='customer_id',
         time_index='time',
@@ -389,12 +457,44 @@ def test_search_empty_labels(transactions, total_spent_fn):
         minimum_data=1,
         num_examples_per_instance=2,
         gap=3,
-        verbose=False,
     )
 
     assert given_labels.empty
 
 
+def test_search_on_empty_labels(transactions):
+    lm = LabelMaker(
+        target_entity='customer_id',
+        time_index='time',
+        labeling_function=lambda ds: None,
+        window_size=2,
+    )
+
+    given_labels = lm.search(
+        transactions,
+        minimum_data=1,
+        num_examples_per_instance=2,
+        gap=1,
+    )
+
+    assert given_labels.empty
+
+
+def test_slice_context(transactions, total_spent_fn):
+    lm = LabelMaker(
+        target_entity='customer_id',
+        time_index='time',
+        labeling_function=total_spent_fn,
+        window_size='1h',
+    )
+
+    for ds in lm.slice(transactions, num_examples_per_instance=-1, verbose=True):
+        assert ds.context.target_entity == 'customer_id'
+        assert isinstance(ds.context.target_instance, int)
+        assert isinstance(ds.context.slice_number, int)
+        assert ds.context.window == ds.context.gap
+
+
 def test_slice_overlap(transactions, total_spent_fn):
     lm = LabelMaker(
         target_entity='customer_id',
@@ -403,9 +503,7 @@ def test_slice_overlap(transactions, total_spent_fn):
         window_size='1h',
     )
 
-    slices = lm.slice(transactions, num_examples_per_instance=2, verbose=True)
-
-    for df in slices:
+    for df in lm.slice(transactions, num_examples_per_instance=2):
         start, end = df.context.window
         is_overlap = df.index == end
         assert not is_overlap.any()
@@ -413,5 +511,5 @@ def test_slice_overlap(transactions, total_spent_fn):
 
 def test_label_type(transactions, total_spent_fn):
     lm = LabelMaker(target_entity='customer_id', time_index='time', labeling_function=total_spent_fn)
-    lt = lm.search(transactions, num_examples_per_instance=1, label_type='discrete', verbose=False)
+    lt = lm.search(transactions, num_examples_per_instance=1, label_type='discrete')
     assert lt.label_type == 'discrete'
